<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Oxwall Foundation (http://www.oxwall.org/foundation).
 * All portions of the code written by Oxwall Foundation are Copyright (c) 2011. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2011 Oxwall Foundation. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * Data Access Object for `base_attachment` table.
 * 
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_system_plugins.base.bol
 * @since 1.0
 */
class BOL_AttachmentDao extends OW_BaseDao
{
    const USER_ID = 'userId';
    const ADD_STAMP = 'addStamp';
    const STATUS = 'status';
    const FILE_NAME = 'fileName';
    const ORIG_FILE_NAME = 'origFileName';
    const SIZE = 'size';
    const BUNDLE = 'bundle';
    const PLUGIN_KEY = 'pluginKey';

    /**
     * Singleton instance.
     *
     * @var BOL_AttachmentDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return BOL_AttachmentDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    protected function __construct()
    {
        parent::__construct();
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'BOL_Attachment';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'base_attachment';
    }

    /**
     * @param string $timeStamp
     * @return array<BOL_Attachment>
     */
    public function findExpiredInactiveItems( $timeStamp )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::STATUS, 0);
        $example->andFieldLessThan(self::ADD_STAMP, $timeStamp);

        return $this->findListByExample($example);
    }

    /**
     * @param integer $userId
     * @return array<BOL_Attachment>
     */
    public function findByUserId( $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::USER_ID, $userId);
        $example->andFieldEqual(self::STATUS, 1);

        return $this->findListByExample($example);
    }

    /**
     * @param string $fileName
     * @return BOL_Attachment
     */
    public function findAttachmentByFileName( $fileName )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::FILE_NAME, $fileName);

        return $this->findObjectByExample($example);
    }

    /**
     * @param string $pluginKey
     * @param string $bundle
     * @return array<BOL_Attachment>
     */
    public function findAttahcmentByBundle( $pluginKey, $bundle )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::BUNDLE, $bundle);
        $example->andFieldEqual(self::PLUGIN_KEY, $pluginKey);
        return $this->findListByExample($example);
    }

    /**
     * @param string $pluginKey
     * @return array<BOL_Attachment>
     */
    public function findAttahcmentByPluginKey( $pluginKey )
    {
        $example = new OW_Example();
        $example->andFieldEqual(self::PLUGIN_KEY, $pluginKey);
        return $this->findListByExample($example);
    }

    /**
     * @param string $bundle
     * @param int $status
     */
    public function updateStatusByBundle( $pluginKey, $bundle, $status )
    {
        $query = "UPDATE `" . $this->getTableName() . "` SET `" . self::STATUS . "` = :status WHERE `".self::PLUGIN_KEY."` = :pk AND `" . self::BUNDLE . "` = :bundle";
        $this->dbo->query($query, array('status' => $status, 'bundle' => $bundle, 'pk' => $pluginKey));
    }
}
